/*
 *  JNIWrapperjnilib.c
 *  JNIWrapper
 *
 *  Created by marco on 16.09.08.
 *  Copyright (c) 2008, __MyCompanyName__. All rights reserved.
 *
 */

#include "JNIWrapper.h"
#include "Service USB Driver/ServiceUSBToolbox.h"

int shared_function(const char *arg) {
    printf("shared_function called with %s\n", arg);
    return 42;
}

JNIEXPORT jint JNICALL Java_JNIWrapper_native_1method(JNIEnv *env, jobject this, jstring arg) {
  /* Convert to UTF8 */
  const char *argutf  = (*env)->GetStringUTFChars(env, arg, JNI_FALSE);

  /* Call into external dylib function */
  jint rc = shared_function(argutf);
  
  /* Release created UTF8 string */
  (*env)->ReleaseStringUTFChars(env, arg, argutf);

  return rc;
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBGetInputValue_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
 return ServiceUSB_GetInputValue((ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier);
}


JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBGetOutputValue_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  return (UInt16)ServiceUSB_GetOutputValue((ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
}


JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBGetEXValue_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  return ServiceUSB_GetEXValue((ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
}


JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBGetEYValue_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  return ServiceUSB_GetEYValue((ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
}


JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBSetOutputValue_1method(JNIEnv *env, jobject this, jint value,jint inDeviceIdentifier,jlong timeout) {
 return (int)ServiceUSB_SetOutputValue( (UInt16) value,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier,(UInt32) timeout);
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBSendVendorRequest_1method(JNIEnv *env, jobject this, jbyte  inVendorRequest , jint inValue, jint inIndex, jint inDeviceIdentifier,jlong timeout) {
 return ServiceUSB_SendVendorRequest(inVendorRequest, inValue, inIndex,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier,(UInt32) timeout);
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBGetBytesInReadBuffer_1method(JNIEnv *env, jobject this, jbyte  inPipeIndexZeroBase , jint inDeviceIdentifier) {
 return  ServiceUSB_GetBytesInReadBuffer( inPipeIndexZeroBase, (ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBSetOutputValueBy16BitMask_1method(JNIEnv *env, jobject this, jstring  inBitMask , jint inDeviceIdentifier) {
  OSStatus Ooops = 0;
  const char *argutf  = (*env)->GetStringUTFChars(env, inBitMask, JNI_FALSE);
  Ooops = ServiceUSB_SetOutputValueBy16BitMask((const ServiceUSB_16BitMaskStringType*)argutf,(ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier,kServiceUSB_DefaultTimeout);
  (*env)->ReleaseStringUTFChars(env, inBitMask, argutf);
  return Ooops;
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBSetOutputValueBy8BitMask_1method(JNIEnv *env, jobject this, jstring  inBitMask , jint inDeviceIdentifier) {
  OSStatus Ooops = 0;
  const char *argutf  = (*env)->GetStringUTFChars(env, inBitMask, JNI_FALSE);
  Ooops = ServiceUSB_SetOutputValueBy8BitMask((const ServiceUSB_8BitMaskStringType*)argutf,(ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier,kServiceUSB_DefaultTimeout);
  (*env)->ReleaseStringUTFChars(env, inBitMask, argutf);
  return Ooops;
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBSetOutputValueBit_1method(JNIEnv *env, jobject this, jbyte  inSymbol , jbyte  inBit, jint inDeviceIdentifier) {
  return ServiceUSB_SetOutputValueBit( inSymbol, inBit,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier,kServiceUSB_DefaultTimeout);
}



JNIEXPORT jstring JNICALL Java_JNIWrapper_ServiceUSBReadBytesFromReadBuffer_1method(JNIEnv *env, jobject this, jbyte  pipe, jint inDeviceIdentifier) {
  /*
  // get as unichar (2 bytes)
  UInt32 BytesToRead;
  jstring *returnString = 0;
  BytesToRead = ServiceUSB_GetBytesInReadBuffer(pipe,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
  if (BytesToRead){
    jchar* theData=(jchar*)malloc(BytesToRead);
    if (theData != nil){
      ServiceUSB_ReadBytesFromReadBuffer(pipe,theData,&BytesToRead,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
      returnString = (jstring*) (*env)->NewString(env,theData,BytesToRead);
      free(theData);
    }
  }
  return (jstring)returnString;*/
  
  // get as null terminated 1 byte character string
  UInt32 BytesToRead;
  jstring *returnString = 0;
  BytesToRead = ServiceUSB_GetBytesInReadBuffer(pipe,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
  if (BytesToRead){
    char* theData=(char*)malloc(BytesToRead+1);
    if (theData != nil){
      ServiceUSB_ReadBytesFromReadBuffer(pipe,theData,&BytesToRead,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
      theData[BytesToRead] = 0;
      returnString = (jstring*) (*env)->NewStringUTF(env,theData);
      free(theData);
    }
  }
  return (jstring)returnString;
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBWriteBytes_1method(JNIEnv *env, jobject this, jbyte  inPipeIndexZeroBase, jstring sendString, jint inDeviceIdentifier, jlong inTimeout) {
  OSStatus Ooops = 0;
  UInt32 inOutDataLen;
  const char *inData  = (*env)->GetStringUTFChars(env, sendString, JNI_FALSE);
  inOutDataLen = strlen(inData);
  Ooops = ServiceUSB_WriteBytes(inPipeIndexZeroBase, (void*) inData, &inOutDataLen, (ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier, inTimeout );
  (*env)->ReleaseStringUTFChars(env, sendString, inData);
  return Ooops;

}

JNIEXPORT jstring JNICALL Java_JNIWrapper_ServiceUSBGetInputValueAs16BitMask_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  char theMask[17];
  ServiceUSB_GetInputValueAs16BitMask( theMask,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier );
  theMask[16] = 0;
  jstring *returnString =  (jstring*) (*env)->NewStringUTF(env,theMask);
  return (jstring)returnString;
}

JNIEXPORT jstring JNICALL Java_JNIWrapper_ServiceUSBGetInputValueAs8BitMask_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  char theMask[9];
  ServiceUSB_GetInputValueAs8BitMask( theMask,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier );
  theMask[9] = 0;
  jstring *returnString =  (jstring*) (*env)->NewStringUTF(env,theMask);
  return (jstring)returnString;
}

JNIEXPORT jchar JNICALL Java_JNIWrapper_ServiceUSBGetInputValueBit_1method(JNIEnv *env, jobject this, jbyte  inBit, jint inDeviceIdentifier) {
  return (jchar) ServiceUSB_GetInputValueBit(inBit,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier );
}

JNIEXPORT jboolean JNICALL Java_JNIWrapper_ServiceUSBTestInputValueBit_1method(JNIEnv *env, jobject this, jbyte  inBit, jint inDeviceIdentifier) {
  return (jboolean) ServiceUSB_TestInputValueBit(inBit,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier ); 
}

JNIEXPORT jboolean JNICALL Java_JNIWrapper_ServiceUSBCompareInputValue16BitMask_1method(JNIEnv *env, jobject this, jstring  inBitMask , jint inDeviceIdentifier) {
  jboolean returnValue;
  const char *argutf  = (*env)->GetStringUTFChars(env, inBitMask, JNI_FALSE);
  returnValue =  ServiceUSB_CompareInputValue16BitMask((const ServiceUSB_16BitMaskStringType*) argutf,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
  (*env)->ReleaseStringUTFChars(env, inBitMask, argutf);
  return returnValue;
}

JNIEXPORT jboolean JNICALL Java_JNIWrapper_ServiceUSBCompareInputValue8BitMask_1method(JNIEnv *env, jobject this, jstring  inBitMask , jint inDeviceIdentifier) {
  jboolean returnValue;
  const char *argutf  = (*env)->GetStringUTFChars(env, inBitMask, JNI_FALSE);
  returnValue =  ServiceUSB_CompareInputValue8BitMask((const ServiceUSB_8BitMaskStringType*) argutf,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier);
  (*env)->ReleaseStringUTFChars(env, inBitMask, argutf);
  return returnValue;
}

#pragma mark Baustelle
JNIEXPORT jintArray JNICALL Java_JNIWrapper_ServiceUSBPulseEngineWhileMouseButton_1method(JNIEnv *env, jobject this, jintArray engineStatus, jintArray InputValueBit, jint inDeviceIdentifier, jlong timeout) {
  int i;
  ServiceUSB_EngineStatusType inOutEngineStatus; // SInt16
  ServiceUSB_InputValueType inInputValueBit; // UInt16

  jint *helperArray = (jint *)malloc(sizeof(jint)*4);
  helperArray = (*env)->GetIntArrayElements(env,engineStatus,JNI_FALSE);
  for (i=0;i<4;i++)
    inOutEngineStatus[i] = (SInt16)helperArray[i];
  for (i=0;i<4;i++)
    inInputValueBit[i] = (UInt16)helperArray[i];
  
  ServiceUSB_PulseEngineWhileMouseButton(inOutEngineStatus,(const ServiceUSB_InputValueType*)inInputValueBit,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier, (UInt32)timeout);
  jintArray *returnArray = (jintArray*)(*env)->NewIntArray(env,4);
  (*env)->SetIntArrayRegion(env, (jintArray)returnArray, 0, 4,  helperArray);
  
  free(helperArray);
  return (jintArray)returnArray ;
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBPulseEngine_1method(JNIEnv *env, jobject this, jintArray engineStatus, jintArray InputValueBit, jint inDeviceIdentifier, jlong timeout) {
  OSStatus Ooops = 0;
  int i;
  ServiceUSB_EngineStatusType inOutEngineStatus; // SInt16
  ServiceUSB_InputValueType inInputValueBit; // UInt16

  jint *helperArray = (jint *)malloc(sizeof(jint)*4);
  helperArray = (*env)->GetIntArrayElements(env,engineStatus,JNI_FALSE);
  for (i=0;i<4;i++)
    inOutEngineStatus[i] = (SInt16)helperArray[i];
  for (i=0;i<4;i++)
    inInputValueBit[i] = (UInt16)helperArray[i];
  free(helperArray);
  
  Ooops = ServiceUSB_PulseEngineWhileMouseButton(inOutEngineStatus,(const ServiceUSB_InputValueType*)inInputValueBit,(ServiceUSB_DeviceIdentifierRef) inDeviceIdentifier, (UInt32)timeout);
  return Ooops;
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBGetDeviceCount_1method(JNIEnv *env, jobject this) {
  return ServiceUSB_GetDeviceCount();
}

JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBGetNextDeviceIdentifier_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
 return (jint)ServiceUSB_GetNextDeviceIdentifier((ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier);
}

JNIEXPORT jstring JNICALL Java_JNIWrapper_ServiceUSBGetDeviceROMVersion_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  char theString[9];
  sprintf(theString,"0x%X",ServiceUSB_GetDeviceROMVersion((ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier));
  jstring *returnString =  (jstring*) (*env)->NewStringUTF(env,theString);
  return (jstring)returnString;
}

JNIEXPORT jstring JNICALL Java_JNIWrapper_ServiceUSBGetUSBDriverVersion_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  char theString[9];
  sprintf(theString,"0x%X",(int) (ServiceUSB_GetUSBDriverVersion((ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier)>>16));
  jstring *returnString =  (jstring*) (*env)->NewStringUTF(env,theString);
  return (jstring)returnString;
}

JNIEXPORT jboolean JNICALL Java_JNIWrapper_ServiceUSBIs16BitDevice_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  return ServiceUSB_Is16BitDevice((ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier);
}

JNIEXPORT jstring JNICALL Java_JNIWrapper_ServiceUSBGetDeviceSerialnumber_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  Str255 theSerialnr;
  char theString[64];
  memset(theSerialnr,0,sizeof(theSerialnr));
  ServiceUSB_GetDeviceSerialnumber(theSerialnr,(ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier);
  sprintf(theString,"%X",(int)&theSerialnr[1]);
  jstring *returnString =  (jstring*) (*env)->NewStringUTF(env,theString);
  return (jstring) returnString;
}

JNIEXPORT void JNICALL Java_JNIWrapper_ServiceUSBReleaseDevice_1method(JNIEnv *env, jobject this, jint inDeviceIdentifier) {
  ServiceUSB_ReleaseDevice((ServiceUSB_DeviceIdentifierRef)inDeviceIdentifier);
}


JNIEXPORT jint JNICALL Java_JNIWrapper_ServiceUSBBubbele_1method(JNIEnv *env, jobject this, jthrowable theObject, jint inDeviceIdentifier) {
  (*env)->Throw(env,(jthrowable)theObject);
    //jint (JNICALL *Throw)
     // (JNIEnv *env, jthrowable obj);
     return 0;
}


 /* int i;
  //ServiceUSB_EngineStatusType inEngineStatus; //SInt16[4]
  jint inEngineStatus[4];
  inEngineStatus[0] = 2;
  inEngineStatus[1] = 4;
  inEngineStatus[2] = 5;
  inEngineStatus[3] = 7;
  
  jsize inBitsArraySize = (*env)->GetArrayLength(env,inBits);
  printf("\nC:inBitsArraySize=%d\n",(int)inBitsArraySize);
  
  //jint inBitsElements[10];
  //char* theData=(char*)malloc(BytesToRead+1);
  jint *inBitsElements = (jint *)malloc(sizeof(jint));
  inBitsElements = (jint *)(*env)->GetIntArrayElements(env,inBits,JNI_FALSE);
  for (i=0;i<4;i++)
    printf("C:inBitsElements[%d]=%d\n",i,(int)inBitsElements[i]);
  inBitsElements[0] = 44;
  
  //jsize (JNICALL *GetArrayLength)
  //    (JNIEnv *env, jarray array);
  
  //jint * (JNICALL *GetIntArrayElements)
  //    (JNIEnv *env, jintArray array, jboolean *isCopy);

  
  jintArray *myArray = (jintArray*)(*env)->NewIntArray(env,10);
  printf("C:neu erstelltes myArray size=%d\n",(int)(*env)->GetArrayLength(env,(jarray)myArray));
  
  
  (*env)->SetIntArrayRegion(env, (jintArray)myArray, 0, 4,  inEngineStatus);
  
  
  //const char *argutf  = (*env)->GetStringUTFChars(env, arg, JNI_FALSE);
  // const char* (JNICALL *GetStringUTFChars)
  //    (JNIEnv *env, jstring str, jboolean *isCopy);
  
 //void (JNICALL *SetIntArrayRegion)
 //     (JNIEnv *env, jintArray array, jsize start, jsize len, const jint *buf);
  
  //jintArray (JNICALL *NewIntArray)
  //    (JNIEnv *env, jsize len);
  */
  


